"""
Bible application
"""
import random
import datetime

import requests

old_testament_books = [
    "Genesis",
    "Exodus",
    "Leviticus",
    "Numbers",
    "Deuteronomy",
    "Joshua",
    "Judges",
    "Ruth",
    "1 Samuel",
    "2 Samuel",
    "1 Kings",
    "2 Kings",
    "1 Chronicles",
    "2 Chronicles",
    "Ezra",
    "Nehemiah",
    "Esther",
    "Job",
    "Psalms",
    "Proverbs",
    "Ecclesiastes",
    "Song of Solomon",
    "Isaiah",
    "Jeremiah",
    "Lamentations",
    "Ezekiel",
    "Daniel",
    "Hosea",
    "Joel",
    "Amos",
    "Obadiah",
    "Jonah",
    "Micah",
    "Nahum",
    "Habakkuk",
    "Zephaniah",
    "Haggai",
    "Zechariah",
    "Malachi"
]

new_testament_books = [
    "Matthew",
    "Mark",
    "Luke",
    "John",
    "Acts",
    "Romans",
    "1 Corinthians",
    "2 Corinthians",
    "Galatians",
    "Ephesians",
    "Philippians",
    "Colossians",
    "1 Thessalonians",
    "2 Thessalonian",
    "1 Timothy",
    "2 Timothy",
    "Titus",
    "Philemon",
    "Hebrews",
    "James",
    "1 Peter",
    "2 Peter",
    "1 John",
    "2 John",
    "3 John",
    "Jude",
    "Revelation"
]

all_books = new_testament_books + old_testament_books
max_chapter = 150
max_verse = 100


def get_verse(book: dict):
    """

    :param book:
    :return:
    """
    book_name = book.get("book_name")
    chapter = 'unknown'
    verse_number = 'unknown'
    verse = 'unknown'
    for key, value in book.get("chapter").items():
        chapter = key
        verse_number = value.get('verse_nr')
        verse = value.get('verse')

    verse_info = {
        "book_name": book_name,
        "chapter": chapter,
        "verse_number": verse_number,
        "verse": verse
    }
    return verse_info


class BibleVerse(object):
    """
    Bible book object
    """
    book: str = None
    chapter: int = None
    verse_number: int = None
    verse: dict = {}


def get_random_verse() -> (str, int, int, str):
    """

    :return: (book, chapter, verse, texture)
    """
    versions = ["kjv", "hsab", "arabicsv"]
    book = random.choice(all_books)
    chapter = str(random.randint(1, max_chapter))
    verse = str(random.randint(1, max_verse))
    results = {"book": book, "chapter": chapter, "verse": verse, "versions": {}}
    for version in versions:
        try:
            response = requests.get(f'https://getbible.net/json?scripture={book}{chapter}:{verse}&version={version}')
            result: dict = eval(response.text[:-1])
            if result.get("type") == "verse":
                verse_dict = get_verse(result.get("book")[0])
                results["versions"][version] = verse_dict["verse"]
        except NameError:
            return get_random_verse()

    return results


def get_bible_verse(book: str = None, chapter: int = None, verse_no: int = None, versions: list = None,
                    retry: int = 5) -> BibleVerse:
    """
    Get a bible verse
    :param retry:
    :param versions:
    :param book:
    :param chapter:
    :param verse_no:
    :return:
    """

    bible_verse = BibleVerse()
    bible_verse.versions = versions if versions is not None else ["kjv", "hsab", "arabicsv"]
    bible_verse.book = book if book is not None else random.choice(all_books)
    bible_verse.chapter = str(chapter) if chapter is not None else str(random.randint(1, max_chapter))
    bible_verse.verse_number = str(verse_no) if verse_no is not None else str(random.randint(1, max_verse))

    for version in bible_verse.versions:
        try:
            response = requests.get(f'https://getbible.net/json?scripture={bible_verse.book}{bible_verse.chapter}:'
                                    f'{bible_verse.verse_number}&version={version}')
            result: dict = eval(response.text[:-1])
            if result.get("type") == "verse":
                verse_dict = get_verse(result.get("book")[0])
                bible_verse.verse[version] = verse_dict["verse"]
        except NameError:
            retry = retry - 1
            if retry <= 0:
                today = datetime.datetime.today()
                return get_bible_verse(book, today.month, today.day, versions, 5)
            else:
                return get_bible_verse(book=book, versions=versions, retry=retry)
    return bible_verse



"""
Stored data
"""
from pathlib import Path
import random

affirmation_file = Path(__file__).parent / 'affirmations.txt'


def get_random_affirmation():
    """
    From all the affirmations in the txt file, return a random affirmation
    :return:
    """
    with affirmation_file.open('r') as affirmations:
        affirmation_list = []
        for affirmation in affirmations:
            affirmation_list.append(affirmation)

    return affirmation_list[random.randint(0, len(affirmation_list) - 1)]



"""
contains all files related to making animations
"""
import random

from asciimatics.exceptions import ResizeScreenError
from asciimatics.screen import Screen
from asciimatics.effects import Stars, Print
from asciimatics.particles import RingFirework, SerpentFirework, StarFirework, PalmFirework, Explosion
from asciimatics.renderers import SpeechBubble, Rainbow, StaticRenderer
from asciimatics.scene import Scene
import sys

_happy_birthday = """  .---.  .---.    ____    .-------. .-------.  ____     __    _______  .-./`) .-------. ,---------. .---.  .---.  ______        ____       ____     __
 |   |  |_ _|  .'  __ `. \  _(`)_ \\  _(`)_ \ \   \   /  /  \  ____  \\ .-.')|  _ _   \\          \|   |  |_ _| |    _ `''.  .'  __ `.    \   \   /  /
 |   |  ( ' ) /   '  \  \| (_ o._)|| (_ o._)|  \  _. /  '   | |    \ |/ `-' \| ( ' )  | `--.  ,---'|   |  ( ' ) | _ | ) _  \/   '  \  \    \  _. /  ' 
 |   '-(_{;}_)|___|  /  ||  (_,_) /|  (_,_) /   _( )_ .'    | |____/ / `-'`"`|(_ o _) /    |   \   |   '-(_{;}_)|( ''_'  ) ||___|  /  |     _( )_ .'  
 |      (_,_)    _.-`   ||   '-.-' |   '-.-'___(_ o _)'     |   _ _ '. .---. | (_,_).' __  :_ _:   |      (_,_) | . (_) `. |   _.-`   | ___(_ o _)'   
 | _ _--.   | .'   _    ||   |     |   |   |   |(_,_)'      |  ( ' )  \|   | |  |\ \  |  | (_I_)   | _ _--.   | |(_    ._) '.'   _    ||   |(_,_)'    
 |( ' ) |   | |  _( )_  ||   |     |   |   |   `-'  /       | (_{;}_) ||   | |  | \ `'   /(_(=)_)  |( ' ) |   | |  (_.\.' / |  _( )_  ||   `-'  /     
 (_{;}_)|   | \ (_ o _) //   )     /   )    \      /        |  (_,_)  /|   | |  |  \    /  (_I_)   (_{;}_)|   | |       .'  \ (_ o _) / \      /      
 '(_,_) '---'  '.(_,_).' `---'     `---'     `-..-'         /_______.' '---' ''-'   `'-'   '---'   '(_,_) '---' '-----'`     '.(_,_).'   `-..-'       """
_fass = """
 ________    ____       .-'''-.     .-''-.    .-_'''-.      ____     .---.  
|        | .'  __ `.   / _     \  .'_ _   \  '_( )_   \   .'  __ `.  \   /  
|   .----'/   '  \  \ (`' )/`--' / ( ` )   '|(_ o _)|  ' /   '  \  \ |   |  
|  _|____ |___|  /  |(_ o _).   . (_ o _)  |. (_,_)/___| |___|  /  |  \ /   
|_( )_   |   _.-`   | (_,_). '. |  (_,_)___||  |  .-----.   _.-`   |   v    
(_ o._)__|.'   _    |.---.  \  :'  \   .---.'  \  '-   .'.'   _    |  _ _   
|(_,_)    |  _( )_  |\    `-'  | \  `-'    / \  `-'`   | |  _( )_  | (_I_)  
|   |     \ (_ o _) / \       /   \       /   \        / \ (_ o _) /(_(=)_) 
'---'      '.(_,_).'   `-...-'     `'-..-'     `'-...-'   '.(_,_).'  (_I_)  
"""

fireworks = [
    (PalmFirework, 25, 30),
    (PalmFirework, 25, 30),
    (StarFirework, 25, 35),
    (StarFirework, 25, 35),
    (StarFirework, 25, 35),
    (RingFirework, 20, 30),
    (SerpentFirework, 30, 35),
]


class AsciiImage(StaticRenderer):
    """
    This class passes in text with all needed attributes to be processed by asciimatics
    """

    def __init__(self, text) -> None:
        """
        :param text: The text string to convert with Figlet.
        """
        super(AsciiImage, self).__init__()
        self._images = [text]


def blast(screen, quantity: int, blast_frame: int, x, y) -> StarFirework or RingFirework or SerpentFirework or \
                                                            PalmFirework:
    """
    yields many fireworks that will all be on the exact same trajectory and timing to make it create a massive blast
    :returns: a StarFirework, RingFirework, SerpentFirework or PalmFirework effect all on the same trajectory and time
    """
    for _ in range(quantity):
        firework, start, stop = random.choice(fireworks)
        yield firework(screen, x, y, 30, start_frame=blast_frame)


def append_fireworks(screen, duration: int) -> StarFirework or RingFirework or SerpentFirework or PalmFirework:
    """
    yields each firework that has to be added to the effects list
    :param screen: the asciimatics screen object
    :param duration: how long you want the fireworks to go on for
    :returns: a StarFirework, RingFirework, SerpentFirework or PalmFirework effect
    """
    width = screen.width
    height = screen.height
    height_range = (height // 8, height * 3 // 4)
    start_frame_range = duration * 5 // 2
    for _ in range(duration):
        firework, start, stop = random.choice(fireworks)
        x_axis = random.randint(0, width)
        y_axis = random.randint(height_range[0], height_range[1])
        yield firework(screen, x_axis, y_axis, random.randint(start, stop),
                       start_frame=random.randint(0, start_frame_range))


def happy_birthday_animation(screen, messages: list = None) -> None:
    """
    the happy birthday animation
    :param messages:
    :param screen:
    :return:
    """
    messages = messages if messages is not None else [
        {'message': _happy_birthday,
         'y-axis': screen.height // 2 - 12,
         'start': 100,
         'end': False,
         'effect': True},
        {'message': _fass,
         'y-axis': screen.height // 2 + 0,
         'start': 100,
         'end': False,
         'effect': True},
        {'message': 'long message long message long message \nlong message long message long message ',
         'y-axis': screen.height // 6 * 5,
         'start': 300,
         'end': 302,
         'effect': False},
        {'message': 'Press x to exit',
         'y-axis': screen.height // 8 * 7,
         'start': 500,
         'end': False,
         'effect': False},
    ]
    scenes = []
    effects = [
        Stars(screen, screen.width),
        Explosion(screen, x=screen.width // 2, y=screen.height // 2, life_time=100)
    ] + list(append_fireworks(screen, 10000))
    has_done_blast = False
    for message in messages:
        message['end'] = message['end'] if message['end'] is not False else 0
        if message['effect'] is True:
            effects.append(Print(screen, Rainbow(screen, AsciiImage(message['message'])), message['y-axis'],
                                 start_frame=message['start'], stop_frame=message['end']))
            if has_done_blast is False:
                effects += list(blast(screen, 10, message['start'] - 30, screen.width // 2, screen.height // 2))
                has_done_blast = True
        else:
            effects.append(Print(screen, SpeechBubble(message['message']), y=message['y-axis'],
                                 start_frame=message['start'], stop_frame=message['end']))
        if message.get("blast", False) is True:
            effects += list(blast(screen, 10, message['end'], screen.width // 2, message['y-axis']))

    scenes.append(Scene(effects, -1))
    screen.play(scenes, stop_on_resize=True)


def default_birthday_animation(screen) -> None:
    """
    the full animation thing
    """
    is_fass_birthday = random.choice([True, False])
    messages = [
        {'message': _happy_birthday,
         'y-axis': screen.height // 2 - 12,
         'start': 100,
         'end': False,
         'effect': True},
        {'message': _fass,
         'y-axis': screen.height // 2 + 0,
         'start': 100,
         'end': False,
         'effect': True},
        {'message': 'Happy birthday Fassey, I love you',
         'y-axis': screen.height // 6 * 5,
         'start': 300,
         'end': 310,
         'blast': True,
         'effect': False},
        {'message': 'Press x to exit',
         'y-axis': screen.height // 8 * 7,
         'start': 500,
         'end': False,
         'effect': False},
    ]
    if is_fass_birthday:
        happy_birthday_animation(screen, messages=messages)
    else:
        print("hello")


def run_default_birthday_animation() -> None:
    """
    Run the birthday animation
    :return:
    """
    try:
        Screen.wrapper(default_birthday_animation)
    except ResizeScreenError:
        run_default_birthday_animation()


if __name__ == '__main__':
    run_default_birthday_animation()



"""
You won't read this but happy birthday

"""
import datetime
import random

import click
from pyfiglet import Figlet


from . import data
from . import bible

fonts = ['3-d', 'alligator', 'alphabet', 'catwalk', 'cosmic', 'doom', 'epic', 'isometric3', 'poison', 'speed']
f = Figlet(font=random.choice(fonts), width=150)


fasega_birthday = datetime.datetime(1998, 8, 18)

today = datetime.datetime.today()

if today.month <= fasega_birthday.month or (today.month <= fasega_birthday.month and today.day <= fasega_birthday.day):
    next_birthday = datetime.datetime(today.year, fasega_birthday.month, fasega_birthday.day)
else:
    next_birthday = datetime.datetime(today.year + 1, fasega_birthday.month, fasega_birthday.day)

until_next_birthday = next_birthday - today


def today_print():
    """
    Based on the day, decide what to print out
    :return:
    """
    if today.day == fasega_birthday.day and today.month == fasega_birthday.month:
        from . import birthdayAnimation
        birthdayAnimation.run_default_birthday_animation()
        return "Happy birthday Xx"
    else:
        return str(until_next_birthday)[:-7]


@click.command()
def cli() -> None:
    """
    \b
    A command line tool to interact with different part of elmer
    This include run details
    """
    click.echo("Hi Fass")
    click.secho(f'{f.renderText(today_print())}', fg="bright_red")
    click.secho(f'Affirmation:\n{data.get_random_affirmation()}', fg="bright_green")
    verse = bible.get_bible_verse()
    click.secho(f'book: {verse.book}\tchapter: {verse.chapter}\tverse no: {verse.verse_number}', fg="magenta")
    for version, scripture in verse.verse.items():
        click.secho(f'version: {version}\n  {scripture}', fg="bright_magenta")
